% This matlab code computes the numbers in table A2 
% in the online appendix to "Quasi-Bayesian model selection" 
% by Atsushi Inoue and Mototsugu Shintani

% Date       Programmer     Description
% 10/21/2013 Atsushi Inoue  Original code
% 12/13/2016 Atsushi Inoue  Revised
% 11/07/2017 Atsushi Inoue  Revised

% Initialization
B = 1000;  % Number of bootstrap replications used to estimate covariance matrix of impulse responses
fid=fopen('tableA2.tex','a');
fprintf(fid,'\\begin{tabular}{ll|cc|ccccc|ccccc}\\hline\n');
fprintf(fid,' & & & & \\multicolumn{5}{c|}{Marginal Likelihood Based} & \\multicolumn{5}{c}{Modified ML Based} \\\\ \n'); 
fprintf(fid,'& & & & \\multicolumn{5}{c|}{on the RWMH Algorithm} & \\multicolumn{5}{c}{on the RWMH Algorithm} \\\\ \\cline{5-14}\n');
fprintf(fid,'& Weight & & Laplace & \\multicolumn{2}{c}{Geweke} & \\multicolumn{2}{c}{SWZ} & CJ & \\multicolumn{2}{c}{Geweke} & \\multicolumn{2}{c}{SWZ} & CJ \\\\ \n');
fprintf(fid,'T & Matrix & HP & Approx.\\ & $\\tau=0.5$ & $\\tau=0.9$ & $q=0.5$ & $q=0.9$ & & $\\tau=0.5$ & $\\tau=0.9$ & $q=0.5$ & $q=0.9$ & \\\\ \\hline\n');
fclose(fid);
M  = 1000;  % Number of Monte Carlo replications
N  = 50000; % Number of MCMC draws; only second half is used to compute quasi-marginal likelihood
options=optimoptions(@fminunc,'Algorithm','quasi-newton','HessUpdate','bfgs','FunvalCheck','on','Display','none');
rng('default');
warning('off')
% Monte Carlo experiment parameters
thetaA       = parameters;
thetaA.value = [0.5 0.5 1 1]; 
thetaA.name  = {'alpha ','kappa ','sigma ','zeta  '}; 
thetaB = thetaA;

for experiment = 1:3
    switch experiment
        case 1
            thetaA.estimation = [0 1 1 0];
            thetaB.estimation = [0 0 1 0];
	    thetaB.value      = [0.5 1 1 1];
            c0A = 1; c1A = 2; c0B = 2; c1B = 4;	
	    indx = [1 1 1 1 1];
        case 2
            thetaA.estimation = [0 0 1 0];
            thetaB.estimation = [0 1 1 0];
            c0A = 2; c1A = 4; c0B = 1; c1B = 2;	
	    indx = [1 1 1 1 1];
        case 3
	    thetaA.estimation = [0 1 1 0];
            thetaB.estimation = [0 0 1 0];
	    thetaB.value      = [0.5 1 1 1];
            c0A = 1; c1A = 2; c0B = 2; c1B = 4;	
	    indx = [0 1 0 1 1];
        case 4
            thetaA.estimation = [0 0 1 0];
            thetaB.estimation = [0 1 1 0];
            c0A = 2; c1A = 4; c0B = 1; c1B = 2;	
	    indx = [0 1 0 1 1];
        case 5
            thetaA.estimation = [1 0 0 1];
            sigma = 1; alpha = 0.5; zeta = 1;
            kappa = (1-alpha)*(1-0.99*alpha)*zeta/alpha;
	    thetaA.value = [alpha kappa sigma zeta];
            thetaB.estimation = [1 0 0 1];
            thetaB.value = [alpha kappa 0.5 zeta];
            c0A = 2; c1A = 4; c0B = 2; c1B = 4;
            indx = [1 1 1 1 1];
        case 6
            thetaA.estimation = [1 0 0 1];
            thetaB.estimation = [1 0 1 1];
            alpha = 0.5; zeta = 1;
            kappa = (1-alpha)*(1-0.99*alpha)*zeta/alpha;
            thetaA.value = [alpha kappa 1 zeta];
            thetaB.value = thetaA.value;
            c0A = 2; c1A = 4; c0B = 1; c1B = 2;
            indx = [1 1 1 1 1];
    end
    nA = sum(thetaA.estimation);
    nB = sum(thetaB.estimation);
    kappa = thetaA.value(2);
    sigma = thetaA.value(3);
    Omega0 = [1 0 -sigma;kappa 1 -sigma*kappa;0 0 1];
    Omega0 = Omega0*Omega0';
    P0     = chol(Omega0)';
    thetaA0 = thetaA.value(thetaA.estimation==1);
    thetaB0 = thetaB.value(thetaB.estimation==1);
    fid = fopen('tableA2.tex','a');
    fprintf(fid,'\\multicolumn{14}{c}{Case %2.0f} \\\\ \\hline\n',experiment); 
    fclose(fid);
    for i=1:3
        switch i
            case 1,
                T=50;
            case 2,
                T=100;
            case 3,
                T=200;
        end
        % Monte Carlo loop
        countdiag    = zeros(1,13);
        countopt     = zeros(1,12);
        mlAdiag      = zeros(1,13);
        mlAopt       = zeros(1,12);
        mlBdiag      = zeros(1,13);
        mlBopt       = zeros(1,12);
        for j=1:M
            y        = randn(T,3)*P0';
            Omegahat = y'*y/T;
            gammahat = [Omegahat(1,1);Omegahat(2,1);Omegahat(3,1);Omegahat(2,2);Omegahat(2,3)];
            gammahat = gammahat(indx==1);
            W = [];
            for k=1:B
                ystar        = y(floor(rand(T,1)*T)+1,:);	
                Omegahatstar = cov(ystar,1); % recentering
                gammahatstar = [Omegahatstar(1,1);Omegahatstar(2,1);Omegahatstar(3,1);Omegahatstar(2,2);Omegahatstar(2,3)];
                W            = [W; gammahatstar(indx==1)'];
            end
            Wdiag = inv(diag(var(W))); % diagonal weighting matrix
            Wopt  = inv(cov(W)); % optimal weighting matrix

            [modeAdiag,mlAdiag(1,1),SigmaAdiag] = findmode(gammahat,indx,options,thetaA,Wdiag,thetaA0);
            if experiment<5
                mlAdiag(1,2) = gelbf(indx,options,thetaA,thetaA0,y);
            end
            mlAdiag(1,3) = (nA/2)*log(2*pi)+mlAdiag(1,1)+0.5*log(det(SigmaAdiag));
            MCMCAdiag = mcmc(c0A,c1A,gammahat,indx,N,SigmaAdiag,thetaA,Wdiag,modeAdiag);
            mlAdiag(1,4:8) = mdd(gammahat,indx,MCMCAdiag,thetaA,Wdiag);
            mlAdiag(1,9:13) = mlAdiag(1,4:8)-(1-1/sqrt(T))*mean(MCMCAdiag(:,1));

            [modeBdiag,mlBdiag(1,1),SigmaBdiag] = findmode(gammahat,indx,options,thetaB,Wdiag,thetaB0);
            if experiment<5
                mlBdiag(1,2) = gelbf(indx,options,thetaB,thetaB0,y);
            end
            mlBdiag(1,3) = (nB/2)*log(2*pi)+mlBdiag(1,1)+0.5*log(det(SigmaBdiag));        
            MCMCBdiag = mcmc(c0B,c1B,gammahat,indx,N,SigmaBdiag,thetaB,Wdiag,modeBdiag);
            mlBdiag(1,4:8) = mdd(gammahat,indx,MCMCBdiag,thetaB,Wdiag);
            mlBdiag(1,9:13) = mlBdiag(1,4:8)-(1-1/sqrt(T))*mean(MCMCBdiag(:,1));

            [modeAopt,mlAopt(1,1),SigmaAopt] = findmode(gammahat,indx,options,thetaA,Wopt,thetaA0);
            mlAopt(1,2) = (nA/2)*log(2*pi)+mlAopt(1,1)+0.5*log(det(SigmaAopt));
            MCMCAopt = mcmc(c0A,c1A,gammahat,indx,N,SigmaAopt,thetaA,Wopt,modeAopt);
            mlAopt(1,3:7)  = mdd(gammahat,indx,MCMCAopt,thetaA,Wopt);
            mlAopt(1,8:12) = mlAopt(1,3:7)-(1-1/sqrt(T))*mean(MCMCAopt(:,1));

            [modeBopt,mlBopt(1,1),SigmaBopt] = findmode(gammahat,indx,options,thetaB,Wopt,thetaB0);
            mlBopt(1,2) = (nB/2)*log(2*pi)+mlBopt(1,1)+0.5*log(det(SigmaBopt));
            MCMCBopt = mcmc(c0B,c1B,gammahat,indx,N,SigmaBopt,thetaB,Wopt,modeBopt);
            mlBopt(1,3:7)  = mdd(gammahat,indx,MCMCBopt,thetaB,Wopt);
            mlBopt(1,8:12) = mlBopt(1,3:7)-(1-1/sqrt(T))*mean(MCMCBopt(:,1));

            for k=1:13
                if mlAdiag(k)>mlBdiag(k)
                     countdiag(k) = countdiag(k) + 1;
                end
            end
            for k=1:12
                if mlAopt(k)>mlBopt(k)
                    countopt(k) = countopt(k) + 1;
                end
            end
        end
        countdiag    = countdiag/M;
        countopt     = countopt/M;

        fid=fopen('tableA2.tex','a');
        fprintf(fid,' %3.0f & Diag & \\multirow{2}{*}{%4.2f} & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f \\\\ \n',T,countdiag(1,2:13));
        fprintf(fid,' & Opt & & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f & %4.2f \\\\ \n',countopt(1,2:12));
        fclose(fid);
    end
    fid = fopen('tableA2.tex','a');
    fprintf(fid,'\\hline\n'); 
    fclose(fid);
end
fid=fopen('tableA2.tex','a');
fprintf(fid,'\\end{tabular}\n');
fclose(fid);
