using Distributions

delta_hess = 1e-5 # step size for numerical Hessians; 1e-8 does not work
delta_pd   = 1e-8 # threshold for eigenvalues 
experiment = 1
H = 2 # maximum horizon 
p = 2 # order of VAR
NK = 5 # number of state variables
T = 200 # sample size 

type parameters
#   name::Array{ASCIIString,1}
    value::Array{Float64,1}
    estimation::Array{Bool,1}
    lb::Array{Float64,1}
    ub::Array{Float64,1}
    prior::Array{Char,1} # N normal,I inverse gamma,G gamma,U uniform,B beta
    mean::Array{Float64,1}
    variance::Array{Float64,1}
    c::Array{Float64,1}
end

@inbounds function AB(theta)

    A = Array(Float64,12,12)
    B = Array(Float64,12,12)
    kappa=theta.value[1]
    bet=theta.value[2]
    tau=theta.value[3]
    phip=theta.value[4]
    phix=theta.value[5]
    rhoz=theta.value[6]
    rhor=theta.value[7]
    sigmaz=theta.value[8]
    sigmar=theta.value[9] 

    A = [0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0;
    1.0 0.0 0.0 0.0 0.0 -phip*(1.0-rhor) -phix*(1.0-rhor) 0.0 0.0 0.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0;
    0.0 1.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0;
    0.0 0.0 1.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0;
    1.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0;
    0.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 1.0 0.0;
    0.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0]
    B = [0.0 0.0 bet  0.0 0.0 -1.0 kappa 0.0 0.0 0.0 0.0 0.0;
    rhor 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0;
    0.0 1.0/tau 1.0/tau -1.0/tau 1.0 0.0 -1.0 0.0 0.0 0.0 0.0 0.0;
    0.0 rhoz 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 1.0 0.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 1.0;
    0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 1.0]

    return A,B
end

@inbounds function goftheta(bc,delta_pd,H,p,theta)
    if bc==true 
        a = Array(Float64,12,12)
        b = Array(Float64,12,12)
        A = Array(Float64,5,5)
        c = Array(Float64,7,5)
        vecA = Array(Float64,4*(1+H))
        (a,b) = AB(theta)
        (c,A,nk) = solab(a,b,5)
        sigmaz = theta.value[8]
        sigmar = theta.value[9]

        B = [sigmar 0.0;0.0 sigmaz;zeros(nk-2,2)]
        C = [c[1,:];1.0 zeros(1,nk-1)]
 
        V = reshape(inv(eye(nk*nk)-kron(A,A))*reshape(B*B',nk*nk,1),nk,nk)
        xx = zeros(nk*p,nk*p)
        xy = zeros(nk*p,nk)
        for i =1:(p-1)
            Ai = A^i
            for j=1:(p-i)
                xx[1+(j-1)*nk:j*nk,1+(i+j-1)*nk:(i+j)*nk] = Ai
            end
            xy[1+(i-1)*nk:i*nk,:] = Ai'
        end
        xy[1+(p-1)*nk:p*nk,:] = (A^p)'
        XX = kron(eye(p),C)*xx*kron(eye(p),V*C')
        XX = XX+kron(eye(p),C*V*C')+XX'
        XY = kron(eye(p),C*V)*xy*C' 
        Bhat = inv(XX)*XY

        Sigma = C*V*C'-XY'*inv(XX)*XY
        Phi = [Bhat';eye(2*(p-1)) zeros(2*(p-1),2)]
        phi = eye(2*p)
        Tau = lufact(Sigma) # LDL decomposition through LU decomposition
        D = diag(Tau[:U])
        D[D.<=delta_pd] = delta_pd
        P = Tau[:L]*diagm(sqrt(D))
        g = zeros(H+1,4)
        for i=0:H
            vecA[1+4*i:4*(i+1)] = reshape(phi[1:2,1:2],4)
            phi = phi*Phi
        end
        g = kron(eye(1+H),kron(P',eye(2)))*vecA
        return g
    else
        a = Array(Float64,12,12)
        b = Array(Float64,12,12)
        A = Array(Float64,5,5)
        c = Array(Float64,7,5)
        vecA = Array(Float64,25*(1+H))
        (a,b) = AB(theta)
        (c,A,nk) = solab(a,b,5)
        sigmaz = theta.value[8]
        sigmar = theta.value[9]
        B = [sigmar 0.0;0.0 sigmaz;zeros(nk-2,2)]
        C = [c[1,:];1.0 zeros(1,nk-1)]
        Ai = eye(5)
        for i=0:H
            vecA[1+25*i:25*(i+1)] = reshape(Ai,25)
            Ai = Ai*A
        end
        g = kron(eye(1+H),kron(B',C))*vecA
        return g
    end
end

function myjacobian(bc,delta,delta_pd,H,indx,p,theta,x0)
    theta.value[theta.estimation] = x0
    g0 = goftheta(bc,delta_pd,H,p,theta)
    g1 = Array{Float64,1}
    g2 = Array{Float64,1}
    n  = sum(theta.estimation)
    G  = Array(Float64,length(indx),n)
    for i=1:n
        x = x0
        x[i] = x0[i]+delta
        theta.value[theta.estimation] = x
        try
            g1 = goftheta(bc,delta_pd,H,p,theta)
        catch
            g1 = g0
        end
        x = x0
        x[i] = x0[i]-delta
        theta.value[theta.estimation] = x
        try
            g2 = goftheta(bc,delta_pd,H,p,theta)
        catch
            g2 = g0
        end
        d = g1-g2
        G[:,i] = d[indx]/(2.0*delta)
    end
    return G 
end

@inbounds function solab(A,B,NK)

    C=schurfact!(complex(A),complex(B))
    select = (abs(diag(C[:S])).>abs(diag(C[:T]))) 
    D=ordschur!(C,select)
    nk = sum(select)
    if nk==NK
        Z21 = real(D[:Z][nk+1:end,1:nk])
        Z11 = real(D[:Z][1:nk,1:nk])
        if rank(Z11)==nk 
            S11 = real(D[:S][1:nk,1:nk])
            T11 = real(D[:T][1:nk,1:nk])
            F = real(Z21*inv(Z11))
            P = real(Z11*inv(S11)*T11*inv(Z11))
        else
            error("Singular Matrix")
        end 
    else
        error("No solution or indeteminacy")
    end

    return F,P,nk
end

theta = parameters(
#   ["kappa ","bet   ","tau   ","phip  ","phix  ","rhoz  ","rhor  ","sigmaz","sigmar"], 
    [0.2,0.99,2.0,1.5,0.5,0.66,0.5,0.005,0.004],
    [true,false,true,true,true,true,true,true,true],
    [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
    [Inf,Inf,Inf,Inf,1.0,1.0,Inf,Inf],
    ['G','U','G','G','G','B','B','I','I'],
    [0.2,0.99,2.0,1.5,0.5,0.66,0.5,0.005,0.004],
    [0.1^2,1.0,0.5^2,0.25^2,0.25^2,0.15^2,0.2^2,0.04^2,0.04^2],
    [0.1,0.3])
experiment = Int
H  = Int
p  = Int
bc = Bool
x0 = theta.value[theta.estimation]
for p in (1,2,4,6,8)
    for H in (1,2,4,6,8)
        for experiment = 1:3
            if experiment==1     
                indx = vec([1:2*(H+1);2*(H+1)+2:4*(H+1)]) # all impulse responses
            elseif experiment==2 
                indx = vec([2*(H+1)+2:4*(H+1)]) # impulse responses to monetary policy shock only
            elseif (experiment==3)|(experiment==6)|(experiment==9) 
                indx = vec([1:2*(H+1)]) # impulse responses to technology shock only 
            end
            for bc in (false,true)
                G = myjacobian(bc,delta_hess,delta_pd,H,indx,p,theta,x0)
		@show([p H experiment bc rank(G)])            
            end
        end
    end
end

