using Distributions, LineSearches, Optim

export criterion, irf, is2_function, is2_function2, myhessian, parameters

type parameters
    value::Array{Float64,1}
    estimation::Array{Bool,1}
    lb::Array{Float64,1}
    ub::Array{Float64,1}
    prior::Array{Char,1} #N normal,I inverse gamma,G gamma,U uniform,B beta
    mean::Array{Float64,1}
    variance::Array{Float64,1}
    c::Array{Float64,1}
end

function is2_function(H,indx,J,pmax,pmin,rng,M,N,S,T,T0,thetaA,thetaB)
    delta_hess = 1e-8 # step size for numerical Hessians
    delta_opt  = 1e-8 # threshold for optimization
    delta_pd   = 1e-8 # threshold for eigenvalues 
    m      = length(indx)
    AIC    = Array{Float64}(pmax-pmin+1)
    c      = Array{Float64,2}(7,5)
    CT     = Array{Float64,2}(5,2)
    flag   = Bool
    fmin   = Float64
    gammahat = Array{Float64}(4*(1+H))
    gammastarmax = Array{Float64,2}(S,4*(1+H))
    Hess   = Array{Float64,2}(8,8)
    i      = 0
    icount = zeros(Int,2,12)
    Laplace = Float64
    lmcmc  = Array{Float64}(div(N,2))
    MCMC   = Array{Float64,2}(8,div(N,2))
    ml     = zeros(4,12)
    n      = Int
    NK     = 5
    p      = Int
    prior  = Bool
    ustar  = Array{Float64,2}(T,2)
    Vhat   = Array{Float64,2}(m,m)
    W      = Array{Float64,2}(m,m)
    Wdiag  = Array{Float64,2}(m,m)
    Wopt    = Array{Float64,2}(m,m)
    xmean   = Array{Float64}(8)
    xmin    = Array{Float64}(8)
    ystar  = Array{Float64,2}(T,2)

    (a,b) = AB(thetaA)
    (c,A,nk,flag) = solab(a,b,NK)
    if flag!=0
        error("Either singular matrix or no solution or indeterminacy")
    end
    kappa = thetaA.value[1]
    tau = thetaA.value[3]
    phip = thetaA.value[4]
    phix = thetaA.value[5]
    rhor = thetaA.value[7]
    sigmaz = thetaA.value[8]
    sigmar = thetaA.value[9]
    AT = A'
    BT = [(1-rhor)*(kappa*phip+phix)*sigmaz/tau  sigmaz 0.0 0.0 0.0;sigmar 0.0 0.0 0.0 0.0]
    C = [c[1:1,:];1.0 zeros(1,4)]
    CT = C'
    while i<M
        # Generate data
        state = zeros(T0+T,nk)
        e = randn(rng,T0+T,2)
        for t=2:T0+T 
             state[t,:] = state[(t-1):(t-1),:]*AT+e[t:t,:]*BT
        end
        y = state[(T0+1):(T0+T),:]*CT
        for p=pmin:pmax
            (Bhat,gammahat,uhat) = irf(0,p,y[pmax+1-p:end,:])
            Sigmahat = uhat'*uhat/Float64(T-pmax-2*p-1)
            AIC[p-pmin+1] = log(det(Sigmahat))+2.0*(2.0+4.0*Float64(p))/Float64(T-pmax)
        end
        phat = indmin(AIC)+pmin-1 
        # Estimate VAR and IRF
        (Bhatmax,gammahat,uhatmax) = irf(H,phat,y)
        # Bootstrap impulse responses
        for j=1:S
            ystar[1:phat,:] = y[1:phat,:]
            xstar = [1 reshape(ystar[(phat:-1:1),:]',1,2*phat)] 
            ustar = uhatmax[rand(rng,(1:(T-phat)),T-phat),:]	
            for k=(phat+1):T
                ystar[k,:] = xstar*Bhatmax+ustar[(k-phat):(k-phat),:]
                xstar      = reshape([1;ystar[k,:];xstar[1,2:end-2]],1,2*phat+1)
            end
            (Bstar,gammastar,ustar) = irf(H,phat,ystar)
            gammastarmax[j,:] = gammastar'
        end
        Vhat  = cov(gammastarmax[:,indx]) 
        Wdiag = inv(diagm(diag(Vhat)))
        Wopt  = inv(Vhat)
        # Run the MH algorithm and estimate MDD
        k = 1
        for prior in (true,false)
            p = phat
            for W in (Wdiag,Wopt)
                for theta in (thetaA,thetaB)
                    flag = false
                    n = sum(theta.estimation)
                    fmin = Inf
                    j = 0
                    xinit = theta.mean[theta.estimation]
                    while (fmin == Inf)|(j<J)
                        x = 2.0*xinit.*rand(rng,n)
                        if x[end]>1.0
                            x[end] = 0.99
                        end
                        df = OnceDifferentiable(x -> criterion(gammahat,H,indx,prior,theta,W,x),x)                            
                        try
                            res = optimize(df,x,theta.lb[theta.estimation],theta.ub[theta.estimation],Fminbox{BFGS}(); optimizer=BFGS(),x_tol=delta_opt,f_tol=delta_opt,iterations=100*n,optimizer_o=Optim.Options(x_tol=delta_opt,f_tol=delta_opt,iterations=100*n))
                            if Optim.minimum(res)<fmin
                                fmin = Optim.minimum(res)
                                xmin[1:n] = Optim.minimizer(res)
                            end  
                        catch txt
                            continue
                        end
                        j += 1
                    end
	            if !prior 
        	        ml[k-4,1] = -fmin
		    end
                    if prior 
                        try 
		        (Hess[1:n,1:n],flag0) = myhessian(delta_hess,gammahat,H,indx,theta,W,xmin[1:n])
                        V = inv(Hess[1:n,1:n])
                        V = eigfact(V)
                        v = real(V[:values])
                        v[v.<=delta_pd] = delta_pd
                        P = real(V[:vectors])*diagm(sqrt.(v))
                        ml[k,2] = 0.5*sum(theta.estimation)*log(2.0*pi)-fmin+0.5*log((det(P))^2.0)
show(*)
                        (lmcmc,MCMC[1:n,:],rng)=mcmc(gammahat,H,indx,N,P,rng,theta,W,xmin[1:n])
show($)
			(ml[k,3:7],rng) = mdd(delta_pd,gammahat,H,indx,lmcmc,MCMC[1:n,:],N,rng,theta,W,-fmin,xmin[1:n])
show(&)
                        ml[k,8:12] = ml[k,3:7]-(1.0-1.0/sqrt(Float64(T)))*mean(lmcmc)
                        catch txt
                            flag = true 
                            break
                        end
             	    end
                    if flag == true
                        break
                    end
                    k += 1
                end
                if flag == true
                    break
                end
            end
            if flag == true
                break
            end
        end
        if flag == true
            continue
        end
        for j=1:2
            for k = 1:12
                if ml[2*j-1,k]>ml[2*j,k]
                    icount[j,k] += 1
                end
            end
        end
        i += 1
    end
    return icount
end

function is2_function2(delta_pd,gammahat,H,indx,M,N,P,rng,theta,W,qmode,xmode)
    i = 1
    lmcmc  = Array{Float64}(div(N,2))
    MCMC   = Array{Float64,2}(length(xmode)+1,div(N,2))
    ml     = Array{Float64}(5)
    mlmat  = Array{Float64}(M,5)
    while i<=M
        try 
            (lmcmc,MCMC,rng)=mcmc(gammahat,H,indx,N,P,rng,theta,W,xmode)
            (ml,rng) = mdd(delta_pd,gammahat,H,indx,lmcmc,MCMC,N,rng,theta,W,qmode,xmode)
	    mlmat[i,:]=ml
            i+=1
        catch(txt)
            continue
        end
    end
    return mlmat
end

@inbounds function AB(theta)
    kappa = theta.value[1]
    bet   = theta.value[2]
    tau   = theta.value[3]
    phip  = theta.value[4]
    phix  = theta.value[5]
    rhoz  = theta.value[6]
    rhor  = theta.value[7]
    I = [2,2,2,4,5,6,7,8,9,10,11,12,12]
    J = [1,6,7,2,6,8,3,1,4,7,11,5,12]
    V = [1.0,-phip*(1.0-rhor),-phix*(1.0-rhor),1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,0.0]
    A = sparse(I,J,V)
    A = full(A)
    I = [1,1,1,2,3,3,3,3,3,4,5,6,7,8,9,10,11,12]
    J = [3,6,7,1,2,3,4,5,7,2,8,9,9,10,10,11,12,12]
    V = [bet,-1.0,kappa,rhor,1.0/tau,1.0/tau,-1.0/tau,1.0,-1.0,rhoz,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
    B = sparse(I,J,V)
    B = full(B)
    return A,B
end

@inbounds function criterion(gammahat,H,indx,prior,theta,W,x)
    g = Array{Float64,2}(4*(H+1),1)
    theta.value[theta.estimation] = x
    (g,flag) = goftheta(H,theta)
    if flag == 0
        d = gammahat-g
        q = 0.5*d[indx]'*W*d[indx]
        r = q[1]
        if prior
            r = r-logprior(theta)
        end
    else
        r = 10.0^8 
    end
    return r
end

@inbounds function goftheta(H,theta)
    a = Array{Float64,2}(12,12)
    b = Array{Float64,2}(12,12)
    (a,b) = AB(theta)
    (c,A,nk,flag) = solab(a,b,5)
    if flag==0 
        vecA = Array{Float64}(nk*nk*(1+H))
        kappa  = theta.value[1]
        tau    = theta.value[3]
        phip   = theta.value[4]
        phix   = theta.value[5]
        rhor   = theta.value[7]
        sigmaz = theta.value[8]
        sigmar = theta.value[9]
        BT = [(1-rhor)*(kappa*phip+phix)*sigmaz/tau sigmaz zeros(1,nk-2);sigmar zeros(1,nk-1)]
        C = [c[1:1,:];1.0 zeros(1,nk-1)]
        Ai = eye(nk)
        vecA[1:nk*nk] = reshape(Ai,nk*nk)
        for i=1:H
            Ai = Ai*A
            vecA[1+nk*nk*i:nk*nk*(i+1)] = reshape(Ai,nk*nk)
        end
        g = kron(eye(1+H),kron(BT,C))*vecA
    else
        g = []
    end
    return g,flag
end

@inbounds function irf(H,p,y)
 
    T = size(y,1)
    X = ones(T-p,1)
    for i=1:p
        X = [X y[p-i+1:T-i,:]]
    end
    Bhat     = inv(X'*X)*X'*y[p+1:T,:]
    uhat     = y[p+1:T,:]-X*Bhat
    Sigmahat = uhat'*uhat/Float64(T-2*p-1)
    if H>0
        Phihat   = [Bhat[2:end,:]'; eye(2*(p-1)) zeros(2*(p-1),2)]
        phihat   = eye(2*p)
        PT       = chol(Sigmahat)
        vecA     = Array{Float64}(4*(1+H))
        vecA[1:4] = reshape(phihat[1:2,1:2],4)
        for i=1:H
            phihat = phihat*Phihat
            vecA[1+4*i:4*(i+1)] = reshape(phihat[1:2,1:2],4)
        end
        gammahat = kron(eye(1+H),kron(PT,eye(2)))*vecA
    else
        gammahat = []
    end
    return Bhat,gammahat,uhat
end

@inbounds function logprior(theta)
    lnp = 0.0
    for i = 1:length(theta.value)
        if theta.estimation[i] == true
            mu = theta.mean[i]
            sigma2 = theta.variance[i] 
            x = theta.value[i]
            if theta.prior[i]=='G' # gamma
                b = sigma2/mu
                a = mu/b
                lnp = lnp+logpdf(Gamma(a,b),x)
            elseif theta.prior[i]=='B' # beta
                a = (1.0-mu)*mu^2.0/sigma2-mu
                b = a/mu-a
                lnp = lnp + logpdf(Beta(a,b),x)
            elseif theta.prior[i]=='I' # inverse gamma
                a = 2.0*mu^2.0/sigma2 + 4.0
                b = mu*(a - 2.0) 
                lnp = lnp - lgamma(a) + log(b^a*(1.0/x)^(a+1.0)*exp(-b/x)) 
            else theta.prior[i]=='U' # uniform
                lnp = lnp + logpdf(Uniform(mu,sigma2),x)
            end
        end
    end
    return lnp
end

@inbounds function logpriorhessian(theta)
    h = Array{Float64}(sum(theta.estimation))
    j = 1
    for i = 1:length(theta.value)
        if theta.estimation[i] == true
            mu = theta.mean[i]
            sigma2 = theta.variance[i] 
            x = theta.value[i]
            if theta.prior[i]=='G' # gamma
                b = sigma2/mu
                a = mu/b
                h[j] = -(a-1.0)/x^2
            elseif theta.prior[i]=='B' # beta
                a = (1.0-mu)*mu^2.0/sigma2-mu
                b = a/mu-a
                h[j] = -(a-1.0)/x^2-(b-1.0)/(1.0-x)^2
            elseif theta.prior[i]=='I' # inverse gamma
                a = 2.0*mu^2.0/sigma2 + 4.0
                b = mu*(a - 2.0) 
                h[j] = (a+1.0)/x^2-2.0*b/x^3
            else theta.prior[i]=='U' # uniform
                h[j] = 0.0
            end
            j += 1
        end
        
    end
    return h
end

@inbounds function mcmc(gammahat,H,indx,N,P,rng,theta,W,x)
    c0     = theta.c[1]
    c1     = theta.c[2]
    L0     = -Inf
    L1     = Float64
    lmcmc  = Array{Float64}(div(N,2))
    MCMC   = Array{Float64}(sum(theta.estimation),div(N,2))
    Nhalf  = div(N,2)
    prior  = true
    u      = rand(rng,N)
    x0     = Float64
    z      = c1*P*randn(rng,sum(theta.estimation),N)
    while L0 == -Inf
        x0 = x+c0*P*randn(rng,size(x))
        L0 = -criterion(gammahat,H,indx,prior,theta,W,x0)
        if L0==-10.0^8
            continue
        end
    end
    for j=1:Nhalf  
        x1 = x0+z[:,j]
    	L1 = -criterion(gammahat,H,indx,prior,theta,W,x1)
        if L1==-10^8
            continue 
        end
        if u[j] < exp(L1-L0)
            x0     = x1  
            L0     = L1
        end
    end
    for j=(Nhalf+1):N
        x1 = x0+z[:,j]
    	L1 = -criterion(gammahat,H,indx,prior,theta,W,x1)
        if L1==-10.0^8
            continue 
        end
        if u[j] < exp(L1-L0)
            x0 = x1  
            L0 = L1
        end
        lmcmc[j-Nhalf] = L0
        MCMC[:,j-Nhalf] = x0
    end
    return lmcmc,MCMC,rng
end

@inbounds function mdd(delta_pd,gammahat,H,indx,lmcmc,MCMC,N,rng,theta,W,qmode,xmode)
    prior = true
    d     = sum(theta.estimation)
    chi25 = quantile(Chisq(d),0.5)
    chi29 = quantile(Chisq(d),0.9)
    HM5   = 0.0
    HM9   = 0.0
    L     = Float64
    Nhalf = div(N,2)
    num   = 0.0
    r     = Array{Float64}(Nhalf)
    SWZ5  = 0.0
    SWZ9  = 0.0
    tauhat5 = 0.0
    tauhat9 = 0.0
    xmean   = mean(MCMC,2)
    qmean = -criterion(gammahat,H,indx,prior,theta,W,xmean)
    if qmean==-10^8
        error("Either singular matrix or no solution or indeterminacy")
    end
    Sigmahat = cov(MCMC')
    Sigmahat = eigfact(Sigmahat)
    v        = real(Sigmahat[:values])
    v[v.<=delta_pd] = delta_pd
    P = real(Sigmahat[:vectors])*diagm(sqrt.(abs.(v)))
    Sigmahat = Sigmahat[:vectors]*diagm(v)*Sigmahat[:vectors]'
    for j=1:Nhalf
        L0 = lmcmc[j]
        x0 = MCMC[:,j:j]
        a = (x0-xmean)'*inv(Sigmahat)*(x0-xmean)
        r[j] = a[1]
    end
    # Harmonic mean estimator
    indx5 = r.<chi25
    indx9 = r.<chi29  
    HM5 = sum(exp.(-0.5*r[indx5]-lmcmc[indx5]))/Float64(Nhalf)
    HM9 = sum(exp.(-0.5*r[indx9]-lmcmc[indx9]))/Float64(Nhalf)
    HM5 = log(0.5)-log(HM5)+0.5*Float64(d)*log(2.0*pi)+0.5*log(det(Sigmahat))
    HM9 = log(0.9)-log(HM9)+0.5*Float64(d)*log(2.0*pi)+0.5*log(det(Sigmahat))
    # The Chib-Jeliazkov estimator
    Sigmatilde = cov(MCMC')+(xmean-xmode)*(xmean-xmode)'
    Sigmatilde = eigfact(Sigmatilde)
    v = real(Sigmatilde[:values])
    v[v.<=delta_pd] = delta_pd
    P = real(Sigmatilde[:vectors])*diagm(sqrt.(abs.(v)))
    Sigmatilde = Sigmatilde[:vectors]*diagm(v)*Sigmatilde[:vectors]'
    for j=1:Nhalf
        L0 = lmcmc[j]
        x0 = MCMC[:,j:j]
        a = (x0-xmode)'*inv(Sigmatilde)*(x0-xmode)
        r[j] = a[1]
    end
    num = sum(min.(exp.(qmode-lmcmc),1.0).*exp.(-0.5*r))
    den = 0.0
    x  = kron(xmode,ones(1,Nhalf))+P*randn(rng,d,Nhalf)
    for j=1:Nhalf
        L = -criterion(gammahat,H,indx,prior,theta,W,x[:,j])
        if L==-10.0^8
            continue
        end
        den = den+min(exp(L-qmean),1.0) 
    end
    CJ = qmode-log(num/den)+(0.5*d)*log(2.0*pi)+0.5*log(det(Sigmatilde))
    # Sims, Waggoner and Zha's r
    for j=1:Nhalf
        L0 = lmcmc[j]
        x0 = MCMC[:,j:j]
        a = (x0-xmode)'*inv(Sigmatilde)*(x0-xmode)
        r[j] = a[1]
    end
    r = sqrt.(r)
    sortedr = sort(r)
    c1  = sortedr[Int(0.01*Nhalf)+1]
    c10 = sortedr[Int(0.10*Nhalf)+1]
    c90 = sortedr[Int(0.90*Nhalf)+1]
    nu  = log(0.1/0.9)/log(c10/c90)
    a   = c1
    b   = c90/0.9^(1.0/nu)
    g   = nu*r.^(nu-1.0)/(b^nu-a^nu)
    ftilde = gamma(0.5*d)*g./(2*pi^(0.5*d)*sqrt(det(Sigmatilde))*r.^Float64(d-1))
    sortedL = sort(lmcmc)
    L5  = sortedL[Int(0.5*Nhalf)+1]
    L9  = sortedL[Int(0.1*Nhalf)+1]
    indx9 = (lmcmc.>L9).&(r.>=a).&(r.<=b)
    indx5 = (lmcmc.>L5).&indx9
    SWZ9 = sum(ftilde[indx9].*exp.(-lmcmc[indx9]))
    SWZ5 = sum(ftilde[indx5].*exp.(-lmcmc[indx5]))
    u = rand(rng,Nhalf)
    z = randn(rng,d,Nhalf)
    s = r'./sqrt.(sum(z.*z,1))
    x = kron(xmean,ones(1,Nhalf))+(ones(d,1)*s).*(P*z)
    for j=1:Nhalf
        L = -criterion(gammahat,H,indx,prior,theta,W,x[:,j])
        if L==-10.0^8
            continue
        end
        if L>L9
            tauhat9 = tauhat9+1.0
            if L>L5
                tauhat5 = tauhat5+1.0
            end
        end
    end
    SWZ5 = -log(SWZ5/tauhat5)
    SWZ9 = -log(SWZ9/tauhat9)
    ml   = [HM5 HM9 SWZ5 SWZ9 CJ]
    return ml, rng
end

@inbounds function myhessian(delta,gammahat,H,indx,theta,W,x0)
    theta.value[theta.estimation] = x0
    (g0,flag0) = goftheta(H,theta)
    if flag0==0
        g1 = Array{Float64,1}
        g2 = Array{Float64,1}
        g3 = Array{Float64,1}
        g4 = Array{Float64,1}
        n  = sum(theta.estimation)
        G  = Array{Float64}(length(indx),n)
        Myhess = Array{Float64}(n,n)
        Myhess = diagm(logpriorhessian(theta))
        for i=1:n
            x = x0
            x[i] = x0[i]+delta
            theta.value[theta.estimation] = x
            (g1,flag) = goftheta(H,theta)
            if flag!=0
                g1 = g0
            end
            x = x0
            x[i] = x0[i]-delta
            theta.value[theta.estimation] = x
            (g2,flag) = goftheta(H,theta)
            if flag!=0
                g2 = g0
            end
            d = g1-g2
            G[:,i] = d[indx]/(2.0*delta)
            x = x0
            x[i] = x0[i]+2.0*delta
            theta.value[theta.estimation] = x
            (g1,flag) = goftheta(H,theta)
            if flag!=0
                g1 = g0
            end
            x = x0
            x[i] = x0[i]-2.0*delta
            theta.value[theta.estimation] = x
            (g2,flag) = goftheta(H,theta)
            if flag!=0
                g2 = g0
            end
            d1 = g1-2.0*g0+g2
            d2 = gammahat-g0
            h = -d1[indx]'*W*d2[indx]/(4.0*delta*delta)
            Myhess[i,i] += h[1]
            if i>1
                for j=1:i-1
                    x = x0
                    x[i] = x0[i]+delta
                    x[j] = x0[j]+delta
                    theta.value[theta.estimation] = x
                    (g1,flag) = goftheta(H,theta)
                    if flag!=0
                        g1 = g0
                    end
                    x[i] = x0[i]-delta
                    x[j] = x0[j]+delta
                    theta.value[theta.estimation] = x
                    (g2,flag) = goftheta(H,theta)
                    if flag!=0
                        g2 = g0
                    end
                    x[i] = x0[i]+delta
                    x[j] = x0[j]-delta
                    theta.value[theta.estimation] = x
                    (g3,flag) = goftheta(H,theta)
                    if flag!=0
                        g3 = g0
                    end
                    x[i] = x0[i]-delta
                    x[j] = x0[j]-delta
                    theta.value[theta.estimation] = x
                    (g4,flag) = goftheta(H,theta)
                    if flag!=0
                        g4 = g0
                    end
                    d1 = g1-g2-g3+g4
                    h = -d1[indx]'*W*d2[indx]/(4.0*delta*delta)
                    Myhess[i,j] += h[1]
                    Myhess[j,i] += h[1]
                end
            end
        end
        Myhess += G'*W*G
    else 
        Myhess = []
    end
    return Myhess,flag0
end

@inbounds function solab(A,B,NK)
    C=schurfact!(complex(A),complex(B))
    select = (abs.(diag(C[:S])).>abs.(diag(C[:T]))) 
    D=ordschur!(C,select)
    nk = sum(select)
    if nk==NK
        Z21 = real(D[:Z][nk+1:end,1:nk])
        Z11 = real(D[:Z][1:nk,1:nk])
        S11 = real(D[:S][1:nk,1:nk])
	if (rank(Z11)==nk)&(rank(S11)==nk) 
	    T11 = real(D[:T][1:nk,1:nk])
            F = real(Z21*inv(Z11))
            P = real(Z11*inv(S11)*T11*inv(Z11))
            flag = 0
        else
            F = []
            P = []
            flag = 1 # Singular matrix
        end 
    else
        F = []
        P = []
        flag = 2 # No solution or indeteminacy
    end
    return F,P,nk,flag
end

