# This Julia code computes the results in table 4 in  
# Atsushi Inoue and Mototsugu Shintani (2017) 
# "Quasi-Bayesian model selection" 

# Date       Programmer     Description
# 12/23/2017 Atsushi Inoue  Revised

# The results reported in the paper are computed using Julia Version 0.6.0.
module IS2_module
    include("is2_module.jl")
end
using IS2_module
@everywhere include("is2_module.jl")

c      = Array{Float64,2}(7,5)
CT     = Array{Float64,2}(5,2)
delta_hess = 1e-8 # step size for numerical Hessians
delta_opt  = 1e-8 # threshold for optimization
delta_pd   = 1e-8 # threshold for eigenvalues 
experiment = Int
filename1  = string("table4a.tex")
filename2  = string("table4b.tex")
filenameA1 = string("tableA9a.tex") 
filenameA2 = string("tableA9b.tex") 
flag = Int
fmin   = Float64
Hess = Array{Float64,2}(8,8)
j = Int
J = 20 # number of initial draws for optimization
N = 50000 # number of draws for the RWMH algorithm
n = Int
NK = 5
results  = Array{Float64,2}(100,5)
results1 = Array{Float64,2}(100,5)
results2 = Array{Float64,2}(100,5)
rng = MersenneTwister(3102009)
rng1 = MersenneTwister(1)
rng2 = MersenneTwister(2)
rng3 = MersenneTwister(3)
rng4 = MersenneTwister(4)
rng5 = MersenneTwister(5)
rng6 = MersenneTwister(6)
rng7 = MersenneTwister(7)
rng8 = MersenneTwister(8)
rng9 = MersenneTwister(9)
rng10 = MersenneTwister(10)
S = 1000 # number of bootstrap replications
T0 = 50 # number of initial observations discarded
theta = parameters(
#   ["kappa ","bet   ","tau   ","phip  ","phix  ","rhoz  ","rhor  ","sigmaz","sigmar"], 
    [0.0245,0.99,1.0,1.5,0.125,0.9,0.75,0.3,0.2],
    [true,false,true,false,true,false,true,false,false],
    [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
    [Inf,1.0,Inf,Inf,Inf,1.0,1.0,Inf,Inf],
    ['G','U','G','G','G','B','B','I','I'],
    [0.0245,0.99,1.0,1.5,0.125,0.9,0.75,0.3,0.2],
    [0.01^2,1.0,0.5^2,0.25^2,0.1^2,0.2^2,0.2^2,0.2^2,0.2^2],
    [1.0,0.3])
xmean  = Array{Float64}(8)
xmin   = Array{Float64}(8)

(a,b) = AB(theta)
(c,A,nk,flag) = solab(a,b,NK)
if flag!=0
    error("Either singular matrix or no solution or indeterminacy")
end
kappa  = theta.value[1]
tau    = theta.value[3]
phip   = theta.value[4]
phix   = theta.value[5]
rhor   = theta.value[7]
sigmaz = theta.value[8]
sigmar = theta.value[9]
AT     = A'
BT     = [(1-rhor)*(kappa*phip+phix)*sigmaz/tau  sigmaz 0.0 0.0 0.0;sigmar 0.0 0.0 0.0 0.0]
C      = [c[1:1,:];1.0 zeros(1,4)]
CT     = C'

s = open(filename1,"a")
@printf(s,"\\begin{tabular}{ll|cccccc}\\hline\n") 
@printf(s,"T & H & \\multicolumn{2}{c}{Geweke} & \\multicolumn{2}{c}{SWZ} & \\multicolumn{2}{c}{CJ} \\\\ \\hline \n")
@printf(s,"\\multicolumn{8}{c}{All impulse responses are used} \\\\ \\hline \n")
close(s)
s = open(filenameA1,"a")
@printf(s,"\\begin{tabular}{ll|cccccccccc}\\hline\n") 
@printf(s,"& & \\multicolumn{4}{c}{Geweke} & \\multicolumn{4}{c}{SWZ} & \\multicolumn{2}{c}{CJ} \\\\ \n")
@printf(s,"T & H & \\multicolumn{2}{c}{\$\\tau=0.5\$} & \\multicolumn{2}{c}{\$\\tau=0.9\$} & \\multicolumn{2}{c}{\$q=0.5\$} & \\multicolumn{2}{c}{\$q=0.9\$} & & \\\\ \\hline \n")
@printf(s,"\\multicolumn{12}{c}{All impulse responses are used} \\\\ \\hline \n")
close(s)
s = open(filename2,"a")
@printf(s,"\\hline\n")
@printf(s,"\\multicolumn{8}{c}{Only impulse responses to the technology shock are used} \\\\ \\hline \n")
close(s)
s = open(filenameA2,"a")
@printf(s,"\\hline\n")
@printf(s,"\\multicolumn{12}{c}{Only impulse responses to the technology shock are used} \\\\ \\hline \n")
close(s)

# Generate data
state = zeros(T0+200,nk)
e = randn(rng,T0+200,2)
for t=2:T0+200 
    state[t,:] = state[(t-1):(t-1),:]*AT+e[t:t,:]*BT
end
y = state[(T0+1):(T0+200),:]*CT

for T in [50,100,200] # sample size 
    pmax = round(Int,5.0*(T/log(T))^0.25) # maximum lag order length of VAR
    ustar  = Array{Float64,2}(T,2)
    ystar  = Array{Float64,2}(T,2)
    for H in [2,4,8] # maximum horizon    
        pmin = H # minimum lag order length of VAR
        AIC    = Array{Float64}(pmax-pmin+1)
        gammahat = Array{Float64}(4*(1+H))
        gammastarmax = Array{Float64,2}(S,4*(1+H))
        indx1 = vec([1:2;4:4*(H+1)]) # all impulse responses
        indx2 = 1
        for i=1:H
            indx2 = [indx2;(1+4*i):(2+4*i)]
        end
        indx2 = vec(indx2) # impulse responses to the technology shock
        for p=pmin:pmax
            (Bhat,gammahat,uhat) = irf(0,p,y[pmax+1-p:T,:])
            Sigmahat = uhat'*uhat/Float64(T-pmax-2*p-1)
            AIC[p-pmin+1] = log(det(Sigmahat))+2.0*(2.0+4.0*Float64(p))/Float64(T-pmax)
        end
        phat = indmin(AIC)+pmin-1 
        # Estimate VAR and IRF
        (Bhatmax,gammahat,uhatmax) = irf(H,phat,y[1:T,:])
        # Bootstrap impulse responses
        for j=1:S
            ystar[1:phat,:] = y[1:phat,:]
            xstar = [1 reshape(ystar[(phat:-1:1),:]',1,2*phat)] 
            ustar = uhatmax[rand(rng,(1:(T-phat)),T-phat),:]	
            for k=(phat+1):T
                ystar[k,:] = xstar*Bhatmax+ustar[(k-phat):(k-phat),:]
                xstar      = reshape([1;ystar[k,:];xstar[1,2:end-2]],1,2*phat+1)
            end
            (Bstar,gammastar,ustar) = irf(H,phat,ystar)
            gammastarmax[j,:] = gammastar'
        end
        Vhat = cov(gammastarmax[:,indx1]) 
        W = inv(Vhat)
        # Run the MH algorithm and estimate MDD
        flag = false
        n = sum(theta.estimation)
        fmin = Inf
        j = 0
        xinit = theta.mean[theta.estimation]
        while (fmin == Inf)|(j<J)
            x = 2.0*xinit.*rand(rng,n)
            if x[end]>1.0
                x[end] = 0.99
            end
            df = OnceDifferentiable(x -> criterion(gammahat,H,indx1,true,theta,W,x),x)                            
            try
                res = optimize(df,x,theta.lb[theta.estimation],theta.ub[theta.estimation],Fminbox{BFGS}(); optimizer=BFGS(),x_tol=delta_opt,f_tol=delta_opt,iterations=100*n,optimizer_o=Optim.Options(x_tol=delta_opt,f_tol=delta_opt,iterations=100*n))
                if Optim.minimum(res)<fmin
                    fmin = Optim.minimum(res)
                    xmin[1:n] = Optim.minimizer(res)
                end  
            catch txt
                continue
            end
            j += 1
        end
        (Hess[1:n,1:n],flag0) = myhessian(delta_hess,gammahat,H,indx1,theta,W,xmin[1:n])
        V = inv(Hess[1:n,1:n])
        V = eigfact(V)
        v = real(V[:values])
        v[v.<=delta_pd] = delta_pd
        P = real(V[:vectors])*diagm(sqrt.(v))
    
        arg1 = Any[delta_pd,gammahat,H,indx1,10,N,P,rng1,theta,W,-fmin,xmin[1:n]]
        arg2 = Any[delta_pd,gammahat,H,indx1,10,N,P,rng2,theta,W,-fmin,xmin[1:n]]
        arg3 = Any[delta_pd,gammahat,H,indx1,10,N,P,rng3,theta,W,-fmin,xmin[1:n]]
        arg4 = Any[delta_pd,gammahat,H,indx1,10,N,P,rng4,theta,W,-fmin,xmin[1:n]]
        arg5 = Any[delta_pd,gammahat,H,indx1,10,N,P,rng5,theta,W,-fmin,xmin[1:n]]
        arg6 = Any[delta_pd,gammahat,H,indx1,10,N,P,rng6,theta,W,-fmin,xmin[1:n]]
        arg7 = Any[delta_pd,gammahat,H,indx1,10,N,P,rng7,theta,W,-fmin,xmin[1:n]]
        arg8 = Any[delta_pd,gammahat,H,indx1,10,N,P,rng8,theta,W,-fmin,xmin[1:n]]
        arg9 = Any[delta_pd,gammahat,H,indx1,10,N,P,rng9,theta,W,-fmin,xmin[1:n]]
        arg10 = Any[delta_pd,gammahat,H,indx1,10,N,P,rng10,theta,W,-fmin,xmin[1:n]]
        result = pmap((args)->is2_function2(args...),Any[arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10])
        for j=1:10
            results[10*(j-1)+1:10*j,:] = result[j,]
        end
        emm = 0
        for j=1:100
            ell = 0
            for k=1:5
                if (abs(results[j,k])<sqrt(prevfloat(Inf)))&(!isnan(results[j,k]))
                    ell += 1
                end
            end
            if ell==5
                emm += 1
                results1[emm,:] = results[j,:]
            end
            if emm==100
                break
            end
        end

        Vhat = cov(gammastarmax[:,indx2]) 
        W = inv(Vhat)
        # Run the MH algorithm and estimate MDD
        flag = false
        fmin = Inf
        j = 0
        xinit = theta.mean[theta.estimation]
        while (fmin == Inf)|(j<J)
            x = 2.0*xinit.*rand(rng,n)
            if x[end]>1.0
                x[end] = 0.99
            end
            df = OnceDifferentiable(x -> criterion(gammahat,H,indx2,true,theta,W,x),x)                            
            try
                res = optimize(df,x,theta.lb[theta.estimation],theta.ub[theta.estimation],Fminbox{BFGS}(); optimizer=BFGS(),x_tol=delta_opt,f_tol=delta_opt,iterations=100*n,optimizer_o=Optim.Options(x_tol=delta_opt,f_tol=delta_opt,iterations=100*n))
                if Optim.minimum(res)<fmin
                    fmin = Optim.minimum(res)
                    xmin[1:n] = Optim.minimizer(res)
                end  
            catch txt
                continue
            end
            j += 1
        end
        (Hess[1:n,1:n],flag0) = myhessian(delta_hess,gammahat,H,indx2,theta,W,xmin[1:n])
        V = inv(Hess[1:n,1:n])
        V = eigfact(V)
        v = real(V[:values])
        v[v.<=delta_pd] = delta_pd
        P = real(V[:vectors])*diagm(sqrt.(v))

        arg1 = Any[delta_pd,gammahat,H,indx2,10,N,P,rng1,theta,W,-fmin,xmin[1:n]]
        arg2 = Any[delta_pd,gammahat,H,indx2,10,N,P,rng2,theta,W,-fmin,xmin[1:n]]
        arg3 = Any[delta_pd,gammahat,H,indx2,10,N,P,rng3,theta,W,-fmin,xmin[1:n]]
        arg4 = Any[delta_pd,gammahat,H,indx2,10,N,P,rng4,theta,W,-fmin,xmin[1:n]]
        arg5 = Any[delta_pd,gammahat,H,indx2,10,N,P,rng5,theta,W,-fmin,xmin[1:n]]
        arg6 = Any[delta_pd,gammahat,H,indx2,10,N,P,rng6,theta,W,-fmin,xmin[1:n]]
        arg7 = Any[delta_pd,gammahat,H,indx2,10,N,P,rng7,theta,W,-fmin,xmin[1:n]]
        arg8 = Any[delta_pd,gammahat,H,indx2,10,N,P,rng8,theta,W,-fmin,xmin[1:n]]
        arg9 = Any[delta_pd,gammahat,H,indx2,10,N,P,rng9,theta,W,-fmin,xmin[1:n]]
        arg10 = Any[delta_pd,gammahat,H,indx2,10,N,P,rng10,theta,W,-fmin,xmin[1:n]]

        result = pmap((args)->is2_function2(args...),Any[arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10])
        for j=1:10
            results[10*(j-1)+1:10*j,:] = result[j,]
        end
        emm = 0
        for j=1:100
            ell = 0
            for k=1:5
                if (abs(results[j,k])<sqrt(prevfloat(Inf)))&(!isnan(results[j,k]))
                    ell += 1
                end
            end
            if ell==5
                emm += 1
                results2[emm,:] = results[j,:]
            end
            if emm==100
                break
            end
        end
		
        mu1 = mean(results1,1)
        sd1 = sqrt.(var(results1,1))
        mu2 = mean(results2,1)
        sd2 = sqrt.(var(results2,1))
        s = open(filename1,"a")
        @printf(s,"%3d & %1d ",T,H)
        for j=1:3
            @printf(s,"& %5.2f & [%5.2f] ",mu1[1+2*(j-1)],sd1[1+2*(j-1)])
        end
        @printf(s,"\\\\ \n")
        close(s)
        s = open(filename2,"a")
        @printf(s,"%3d & %1d ",T,H)
        for j=1:3
            @printf(s,"& %5.2f & [%5.2f] ",mu2[1+2*(j-1)],sd2[1+2*(j-1)])
        end
        @printf(s,"\\\\ \n")
        close(s)
	s = open(filenameA1,"a")
	@printf(s,"%3d & %1d ",T,H)
        for j=1:5
             @printf(s,"& %5.2f & [%5.2f] ",mu1[j],sd1[j])
        end
        @printf(s,"\\\\ \n")
        close(s)
	s = open(filenameA2,"a")
	@printf(s,"%3d & %1d ",T,H)
        for j=1:5
            @printf(s,"& %5.2f & [%5.2f] ",mu2[j],sd2[j])
        end
        @printf(s,"\\\\ \n")
        close(s)
    end
end
s = open(filename2,"a")
@printf(s,"\\hline\n") 
@printf(s,"\\end{tabular}\n") 
close(s)
s = open(filenameA2,"a")
@printf(s,"\\hline\n") 
@printf(s,"\\end{tabular}\n") 
close(s)
