function [Zu]=cal_moment(Y,X,Z,para,para_fix,model,denominator,para_estim)

%% preparation
Z1=Z.Z1;
Z2=Z.Z2;
rho=Z.rho;
lead=X.lead;

n_inst=size(Z1,2);
n_inst2=0;
T=size(Z1,1);

para(para_estim==0)=para_fix(para_estim==0);

% parameter
omega=para(1);
lambda=para(2);
Pistar=1+para(3)/100; % gross rate
theta=para(4); %para_fix(1);
alpha=para(5);%para(5); %para_fix(2);
beta=para(6);%para_fix(3);

% data

log_Pi_t=X.log_Pi_t;
log_Pi_b1=X.log_Pi_b1;
log_Pi_f1=X.log_Pi_f1;
ulchat_t =X.ulchat_t;

log_Pi_bs=X.log_Pi_bs;
log_Pi_bs_b1=X.log_Pi_bs_b1;
log_Pi_fs=X.log_Pi_fs;
gyhat_fs =X.gyhat_fs;
rhat_fs  =X.rhat_fs;


pihat_t = log_Pi_t - 100*log(Pistar);
pihat_f1= log_Pi_f1 - 100*log(Pistar);
pihat_b1= log_Pi_b1 - 100*log(Pistar);
pihat_bs= log_Pi_bs - 100*log(Pistar);
pihat_bs_b1= log_Pi_bs_b1 - 100*log(Pistar);
pihat_fs= log_Pi_fs - 100*log(Pistar);


%% useful definition

A=lambda*Pistar^(theta-1);
A2=lambda*Pistar^(theta);
phi=A+omega*(1-A*(1-beta*Pistar));

% coefficients of GNKPC
Gb=omega/phi;
Gf=beta*A2/phi;
K=(1-omega)*(1-A)*(1-beta*A2)/phi;
Kf=(Pistar-1)*(1-omega)*(1-A)/phi;
Kd=theta*A*(Pistar-1)/(1-A);

powers.f=[lead.f2:-1:1]';
powers.b=[1:1:lead.b2]';

DF1=(ones(lead.b2,1)*lambda*Pistar^theta).^(powers.b);
DF2=(ones(lead.f2,1)*beta*lambda*Pistar^(theta-1)).^(powers.f);

% coefficients of NKPC

phi1=lambda+omega*(1-lambda*(1-beta));

Gb1=omega/phi1;
Gf1=beta*lambda/phi1;
K1 =(1-lambda)*(1-beta*lambda)*(1-omega)/(phi1);

% coefficients of Cogley-Sbordone type GNKPC

Ac=lambda*Pistar^((theta-1)*(1-omega));
phic = 1+omega*beta*Pistar^(1-omega);

Gbc=omega/phic;
Gfc=beta*Pistar^(1-omega)/phic;
Kc=(1-Ac)*(1-beta*lambda*Pistar^(theta*(1-omega)))/(Ac*phic);
Kfc=(Pistar^(1-omega)-1)*(1-Ac)/(Ac*phic);
Kdc=theta*Ac*(Pistar^(1-omega)-1)/(1-Ac);

DFc1=(ones(lead.b2,1)*lambda*Pistar^(theta*(1-omega))).^(powers.b);
DFc2=(ones(lead.f2,1)*beta*lambda*Pistar^((theta-1)*(1-omega))).^(powers.f);

% coefficients of NKPC with partial price indexation to lagged inflation
phic1=1+omega*beta;

Gbc1=omega/phic1;
Gfc1=beta/phic1;
Kc1 =(1-lambda)*(1-beta*lambda)/(lambda*phic1);

%% standard version
if denominator==0

%% GNKPC model
if model==1||model==3||model==4
    
if model==3
    Kd=0;
elseif model==4
    Kf=0;
end

ut=pihat_t - Gb/(1+K*Kd)*pihat_b1 - Gf/(1+K*Kd)*pihat_f1 - K/(1+K*Kd)*ulchat_t...
    + K*Kd/(1+K*Kd)*pihat_bs*DF1...
    - Kf/(1+K*Kd)*(gyhat_fs + theta*pihat_fs - rhat_fs)*DF2;
%% NKPC
elseif model==2
    
ut=pihat_t - Gb1*pihat_b1 - Gf1*pihat_f1 - K1*ulchat_t;

%% Cogley-Sbordone type GNKPC
elseif model==5||model==7||model==8
    
if model==7
    Kdc=0;
elseif model==8
    Kfc=0;
end

CS=1+Kc*Kdc+omega*Kfc*beta*Ac*(theta-1); % denominator
    
ut=pihat_t - (Gbc+Kc*Kdc*omega)/CS*pihat_b1 - Gfc/CS*pihat_f1 - Kc/CS*ulchat_t...
    + Kc*Kd/CS*(pihat_bs-omega*pihat_bs_b1)*DFc1...
    - Kfc/CS*(gyhat_fs + (theta-omega*beta*Ac*(theta-1))*pihat_fs - rhat_fs)*DFc2;
%% NKPC with partial price indexation to lagged inflation
elseif model==6

ut=pihat_t - Gbc1*pihat_b1 - Gfc1*pihat_f1 - Kc1*ulchat_t;

end % end model

    
end % end denominator

%% moment condition

Zu=Z1.*repmat(ut,1,n_inst);

% additional equation
%if (model==1)&&(not(isempty(Z2)))
if not(isempty(Z2))
    n_inst2=size(Z2,2);

    vt= log_Pi_t - 100*log(Pistar);

    Zv=Z2.*repmat(vt,1,n_inst2);
    Zu=[Zu,Zv];
end    
