function [X,Y,Z]=data_load(sample_period, US, cpi, IV, rho, lead)

%% data sample period

% preparing instruments

% for US
if US==1
    rawData = xlsread('USdata0511st.xlsx','US_st');
elseif US==0
    rawData = xlsread('JPdata0512st.xlsx','JP_st');
end

data     =rawData(sample_period(1)-lead.b2:sample_period(2)+lead.f2,:);
data_b1     =rawData(sample_period(1)-lead.b2-1:sample_period(2)+lead.f2-1,:);
data_sampleperiod=rawData(sample_period(1):sample_period(2),:);

% get sample mean for demean
mean_log_gy=mean(data_sampleperiod(:,19));
mean_log_ulc=mean(data_sampleperiod(:,3));
mean_log_l=mean(data_sampleperiod(:,4));
mean_log_r=mean(data_sampleperiod(:,6));

% get data
log_gy_t=data(:,19);
log_ulc_t=data(:,3);
log_l_t=data(:,4);log_r_t=data(:,6);

% linear detrend for log_y_t
quarter_t=data_sampleperiod(:,1);
one1=ones(size(data_sampleperiod,1),1);
log_y_t=data_sampleperiod(:,5);
beta_def =([one1, quarter_t]'*[one1, quarter_t])\([one1, quarter_t]'*log_y_t);

log_y_t=data(:,5);
quarter_t=data(:,1);
one1=ones(size(data,1),1);
predic_def= [one1, quarter_t]*beta_def;
yhat_t = log_y_t - predic_def; 


% preparing instruments

% inflation
log_Pi_t=data(:,25);
lag_log_Pi=lagged_var(log_Pi_t,lead.f2,lead.b2);
pi_iv=lag_log_Pi(:,lead.f2+2:lead.f2+5);

% for CS type GNKPC
log_Pi_t_b1=data_b1(:,25);
lag_log_Pi_b1=lagged_var(log_Pi_t_b1,lead.f2,lead.b2);

% output growth
gyhat_t=log_gy_t - mean_log_gy;
lag_gyhat_t=lagged_var(gyhat_t,lead.f2,lead.b2);
gyhat_iv=lag_gyhat_t(:,lead.f2+2:lead.f2+5);

% demeaned interest rate
rhat_t=log_r_t - mean_log_r;
lag_rhat_t=lagged_var(rhat_t,lead.f2,lead.b2);
rhat_iv=lag_rhat_t(:,lead.f2+2:lead.f2+5);

% ulc
ulchat_t=log_ulc_t - mean_log_ulc; % demean
lag_ulchat_t=lagged_var(ulchat_t,lead.f2,lead.b2);
ulchat_iv=lag_ulchat_t(:,lead.f2+2:lead.f2+5);

% change in labor
diff_lhat_t=data(:,16);
lag_diff_lhat=lagged_var(diff_lhat_t,lead.f2,lead.b2);
diff_lhat_iv=lag_diff_lhat(:,lead.f2+2:lead.f2+5);

% change in interest rate
diff_rhat_t=data(:,23);
lag_diff_rhat=lagged_var(diff_rhat_t,lead.f2,lead.b2);
diff_rhat_iv=lag_diff_rhat(:,lead.f2+2:lead.f2+5);

% demeaned labor
lhat_t=log_l_t - mean_log_l;
lag_lhat_t=lagged_var(lhat_t,lead.f2,lead.b2);
lhat_iv=lag_lhat_t(:,lead.f2+2:lead.f2+5);

% linear detrended output
lag_yhat_t=lagged_var(yhat_t,lead.f2,lead.b2);
yhat_iv=lag_yhat_t(:,lead.f2+2:lead.f2+5);

% change in inflation
diff_pihat_t=data(:,10);
lag_diff_pihat_t=lagged_var(diff_pihat_t,lead.f2,lead.b2);
diff_pi_iv=lag_diff_pihat_t(:,lead.f2+2:lead.f2+5);

% wage growth
dw=data(:,8); 
lag_dw=lagged_var(dw,lead.f2,lead.b2);
dw_iv=lag_dw(:,lead.f2+2:lead.f2+5);

% instruments only for US
if US==1

LSspread=data(:,7);
dc=data(:,9); 

lag_LSspread=lagged_var(LSspread,lead.f2,lead.b2);
lag_dc=lagged_var(dc,lead.f2,lead.b2);

LSS_iv=lag_LSspread(:,lead.f2+2:lead.f2+5);
dc_iv=lag_dc(:,lead.f2+2:lead.f2+5);

end

% variables for estimation
X.log_Pi_t=lag_log_Pi(:,lead.f2+1);
X.log_Pi_b1=lag_log_Pi(:,lead.f2+2);
X.log_Pi_f1=lag_log_Pi(:,lead.f2);
X.ulchat_t =lag_ulchat_t(:,lead.f2+1);

X.log_Pi_bs   =   lag_log_Pi(:,lead.f2+2:lead.f2+1+lead.b2);
X.log_Pi_bs_b1=lag_log_Pi_b1(:,lead.f2+2:lead.f2+1+lead.b2);
X.log_Pi_fs=lag_log_Pi(:,1:lead.f2);
X.gyhat_fs =lag_gyhat_t(:,1:lead.f2);
X.rhat_fs  =lag_rhat_t(:,2:lead.f2+1);

%{

%% get data
data=rawData(sample_period(1):sample_period(2),:); 

% change in inflaton (t to t+3)
diff_pihat_t=data(:,10);
diff_pihat_f1=data(:,11);
diff_pihat_f2=data(:,12);
diff_pihat_f3=data(:,13);

% change in ULC (t+1 and t+2)
diff_ulchat_f1=data(:,14);
diff_ulchat_f2=data(:,15);

% change in labor (t+1 and t+2)
diff_lhat_t = data(:,16);
diff_lhat_f1=data(:,17);
diff_lhat_f2=data(:,18);

% output growth (t to t+2)
log_gy_t=data(:,19);
log_gy_f1=data(:,20);
log_gy_f2=data(:,21);

% change in output growth (
diff_gyhat_f2=data(:,22);
diff_rhat_t=data(:,23);
diff_rhat_f1=data(:,24);

% level of inflation (t-1 to t+1)
log_Pi_t=data(:,25);
log_Pi_b1=data(:,26);
log_Pi_f1=data(:,27);

% level of ULC (for NKPC)
log_ulc_t=data(:,3);

% demean
gyhat_f1=log_gy_f1 - mean_log_gy;
gyhat_f2=log_gy_f2 - mean_log_gy;
ulchat_t = log_ulc_t - mean_log_ulc;


%{
quarter_t=data(:,1); 
log_Pi=data(:,2);% (net) inflation rate, quarterly, percent
log_ulc=data(:,3); % 100*log(labor share)
log_l=data(:,4);  % 100*log(labor)
log_y=data(:,5); % 100*log(real GDP per capita)
log_r=data(:,6); % (net) nominal interest rate, quarterly, percent

if US==1
LSspread=data(:,7); % long-short spread, quarterly, percent 
dw=data(:,8); % wage growth, quarterly, percent
dc=data(:,9); % commodity inflation, quarterly, percent
end

one1=ones(size(data,1),1);

% for Japan
elseif US==0
rawData = xlsread('JPdata0113st.xlsx','JP_st');

data=rawData(sample_period(1)-4:sample_period(2)+3,:); 

quarter_t=data(:,1); 
log_Pi=data(:,2);% (net) inflation rate, quarterly, percent
log_ulc=data(:,3); % 100*log(labor share)
log_l=data(:,4);  % 100*log(labor)
log_y=data(:,5); % 100*log(real GDP per capita)
log_r=data(:,6); % (net) nominal interest rate, quarterly, percent

one1=ones(size(data,1),1);

end

%% data arrangement

% creating lagged variables
lag_log_Pi=lagged_var(log_Pi,3,5);
lag_log_ulc=lagged_var(log_ulc,3,5);
lag_log_l=lagged_var(log_l,3,5);
lag_log_y=lagged_var(log_y,0,1);
lag_log_r=lagged_var(log_r,3,5);

if US==1
lag_LSspread=lagged_var(LSspread,3,5);
lag_dw=lagged_var(dw,3,5);
lag_dc=lagged_var(dc,3,5);
end

lag_log_l2=lagged_var(log_l,0,1);
lag_log_r2=lagged_var(log_r,0,1);
diff_log_l=lag_log_l2(:,1)-lag_log_l2(:,2);
diff_log_r=lag_log_r2(:,1)-lag_log_r2(:,2);

lag_diff_log_l=lagged_var(diff_log_l,3,4);
lag_diff_log_r=lagged_var(diff_log_r,3,4);

diff_log_y=lag_log_y(:,1)-lag_log_y(:,2);

lagg_diff_log_y=lagged_var(diff_log_y,3,4);

% pi used in NKPC estimation
log_Pi_t=lag_log_Pi(:,4);
log_Pi_f1=lag_log_Pi(:,3);
log_Pi_b1=lag_log_Pi(:,5);


% creating differenced variables

diff_pihat_t =lag_log_Pi(:,4)-lag_log_Pi(:,5);
diff_pihat_f1=lag_log_Pi(:,3)-lag_log_Pi(:,4);
diff_pihat_f2=lag_log_Pi(:,2)-lag_log_Pi(:,3);
diff_pihat_f3=lag_log_Pi(:,1)-lag_log_Pi(:,2);

diff_ulchat_f1=lag_log_ulc(:,3)-lag_log_ulc(:,4);
diff_ulchat_f2=lag_log_ulc(:,2)-lag_log_ulc(:,3);

diff_lhat_f1=lag_log_l(:,3)-lag_log_l(:,4);
diff_lhat_f2=lag_log_l(:,2)-lag_log_l(:,3);

diff_rhat_f1=lag_log_r(:,3)-lag_log_r(:,4);

log_gy_t=lag_log_y(:,1)-lag_log_y(:,2);
lag_log_gy=lagged_var(log_gy_t,3,4);

log_gy_f1=lag_log_gy(:,3);
log_gy_f2=lag_log_gy(:,2);

diff_gyhat_f2=log_gy_f2-log_gy_f1;


% preperation for demean
lag_log_y_forDemean=lagged_var(log_y,0,1);
lag_log_ulc_forDemean=lagged_var(log_ulc,0,1);

% Demeaned variables
gyhat_f1=log_gy_f1 - mean(lag_log_y_forDemean(:,1)-lag_log_y_forDemean(:,2));
gyhat_f2=log_gy_f2 - mean(lag_log_y_forDemean(:,1)-lag_log_y_forDemean(:,2));
ulchat_t=lag_log_ulc(:,4) - mean(lag_log_ulc_forDemean(:,1)-lag_log_ulc_forDemean(:,2));
%}


%% Variables for estimation

% level of inflation
X.log_Pi_t=log_Pi_t;
X.log_Pi_f1=log_Pi_f1;
X.log_Pi_b1=log_Pi_b1;

% for deterministic-trend GNKPC
X.log_Pi_f2=lag_log_Pi(:,2);

% for GNKPC 
X.diff_pihat_t=diff_pihat_t;
X.diff_pihat_f1=diff_pihat_f1;
X.diff_pihat_f2=diff_pihat_f2;
X.diff_pihat_f3=diff_pihat_f3;

X.diff_ulchat_f1=diff_ulchat_f1;
X.diff_ulchat_f2=diff_ulchat_f2;

X.diff_lhat_f1=diff_lhat_f1;
X.diff_lhat_f2=diff_lhat_f2;

X.diff_rhat_f1=diff_rhat_f1;

X.diff_gyhat_f2=diff_gyhat_f2;

X.gyhat_f1=gyhat_f1;
X.gyhat_f2=gyhat_f2;

% for NKPC
X.ulchat_t=ulchat_t;

% other possible variables

X.ulchat_f1=lag_ulchat_t(:,2);

X.yhat_t=lag_yhat_t(:,3);
X.yhat_f1=lag_yhat_t(:,2);

X.lhat_t=lag_lhat_t(:,3);
X.lhat_f1=lag_lhat_t(:,2);

X.rhat_t=lag_rhat_t(:,3);
X.rhat_f1=lag_rhat_t(:,2);


% for convenience
Y=log_Pi_t; 
%}

Y=lag_log_Pi(:,lead.f2+1);

% for obtaining sample mean of inflation
if US==1
    Y_fullsample=mean(rawData(77:280,25));
elseif US==0
    Y_fullsample=mean(rawData(47:187,25));
end

eval(char(IV))

Z.Z1=Z1;
Z.Z2=Z2;

Z.Y_fullsample=Y_fullsample;
Z.rho=rho;% dummy

X.lead=lead;