clear all
tic

global prior_set

graphing = 1; % if =1 then graphs will be made.
excel_output = 0 ; %if =1 then an excel output file will be made.

% setting for truncation version

lead.b=0;
lead.f=0;

if lead.b<4
    lead.b2=4;
else
    lead.b2=lead.b;
end

if lead.f<1
    lead.f2=1;
else
    lead.f2=lead.f;
end

%%%%%%%%%%%%%%%%%%%%%%
%% common settings
%%%%%%%%%%%%%%%%%%%%%%

%% (1) save folder name
today1=['w50lambda50nkpc']; % this character is added to the top of the name of saved files;

%% making folder for results
%mkdir(today1)

%% (2) number of MCMC iteration
K=10; % this is burn-in sample size. 20*K is number of MCMC iteration after burn-in. 

%% (3) denominator specification
denominator=0; % =0 for Kurozumi's specification. 
               % =1 for specifiation where all denominators in the model are unity.
               % =2 for specification where coefficient on pi_t is unity.
% Note: the current version accepts only "denominator = 0".

%% (4) country

US=1; % =1 for US data, =0 for Japan

if US==1
    today1=[today1,'US'];
else
    today1=[today1,'JP'];
end


%% (5) price index
if US==1
cpi=0; % =1 for PCE deflator(US).

elseif US==0
cpi=1; % =1 for CPI (JP).

end

%% (6) number of lags of serial correlation consdered in optimal weighting matrix.
NWlags=-1; % number of lags for Newy-West weighting matrix.
          % if =-1, the code implements automatic bandwith selection w/ Bertlett Kernel.
          % if =-2, the code implements automatic bandwith selection w/ Quadratic spectral kernel. 
%(caution) if iniWM=0, NWlags must be 0.

%% (7) CU or 2step GMM
TSGMM=1; %=1 if 2step GMM, =0 CU-GMM

%% (8) select initial (1st step) weighting matrix 
iniWM=0;
% =1 if using 'identity' matrix for 1st step weighting matrix 
% =0 if using 'instrument*instrument' matrix for 1st step WM
% =-1 if using demeaned 'instrument*instrument' matrix.

%% (9) model
% set model to be estimated.
% =1 ulc-specificaton GNKPC
% =2 ulc-specification NKPC
% =3 ulc-specification GNKPC without Kappa_d term
% =4 ulc-specification GNKPC without Kappa_f term

% =5 ulc-specificaton Cogley-Sbordone type GNKPC
% =6 ulc-specification NKPC with partial indexation to lagged inflation
% =7 ulc-specification Cogley-Sbordone type GNKPC without_Kappa_d term
% =8 ulc-specification Cogley-Sbordone type GNKPC without Kappa_f term

start_estim_model= 2; %x 
end_estim_model=2;%start_estim_model; %y
%--> from model x to model y are estimated continuously.


%% (10) sample period
% set data sample to be used
start_estim_period=1; %x

if US==1
    end_estim_period=2; %y
elseif US==0
    end_estim_period=4; %y
end
%end_estim_period=start_estim_period;
%--> from sample x to sample y are used for each period estimation.

% for definition of sample period, see 'sample_period_definition.m'

%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%

mkdir(today1) % making folder for results



%% (11) adjustment parameter for MCMC
% -- element(x,y) indicates an adjustment parameter for model x, sample y. 
adjust_c=[0,0,0;
          2.6, 2.6, 2.4;
          0,0,0;
          0,0,0;
          0,0,0;
          0,0,0;
          0,0,0;
          0,0,0];

    %adjust_c=adjust_c*1.2;

%% (12) parameters of prior distribution.
% set 1 in the first column if the corresponding parameter is to be
% estimated, otherwise set 0 and put fixed value in third column. 
% Second column is distribution type. 
% For beta(B), gamma(G), normal(N), inverse gamma(InvG), or variable treansformed gamma(VTG) distribution,
% third and fourth column indicate prior means and prior stds, respectively.
% For uniform(U) distribution, third and fourth columns indicates lower and
% upper bounds of uniform distribution. 
% VTG is special distribution for time preference parameter(beta).

% for US
if US==1
prior_set=...
    {1, 'B'   0.50  , 0.1  ; % omega, beta(B) or uniform(U) distribution
     1, 'B'   0.50  , 0.1  ; % lambda, beta(B) or uniform(U) distribution
     1, 'N'   0   , 1.5/4;% trend inflation, see the next part, normal(N) or uniform(U) distribution
     0, 'G'   9.32, 1  ; % theta, gamma(G) or uniform(U) distribution
     0, 'B'   1, 0.1; % alpha, beta(B) or uniform(U) distribution
     0, 'VTG' 0.99, 0.1}; % beta (Gamma dist. w/ variable transformation)

%for JAPANESE data
elseif US==0

prior_set=...
    {1, 'B'   0  , 0.1  ; % omega, beta(B) or uniform(U) distribution
     1, 'B'   0.5  , 0.1  ; % lambda, beta(B) or uniform(U) distribution
     1, 'N'   0    , 1.5/4;% trend inflation, see the next part, normal(N) or uniform(U) distribution
     0, 'G'   7.67 , 1  ; % theta, gamma(G) or uniform(U) distribution
     0, 'B'   1  , 0.1; % alpha, beta(B) or uniform(U) distribution
     0, 'VTG' 0.99 , 0.1}; % beta (Gamma dist. w/ variable transformation)
end
%}

%% sample mean vs full-sample mean for prior mean of trend inflation

fullsamplemean=1; %if use full-sample mean of inflation as prior mean, set =1

if fullsamplemean==1
    treat_meanPi={'para_pri(3,1)=mean(Y_fullsample);para_fix(3)=mean(Y_fullsample);'}; % if using sample mean as prior mean for trend inflation.
else
    treat_meanPi={'para_pri(3,1)=mean(Y);para_fix(3)=mean(Y);'}; % if using sample mean as prior mean for trend inflation.
end

treat_iniPi={''};%para_ini(3)=mean(Y)'};
% modify treat_meanPi={';'} and treat_iniPi={';'}; if not using sample mean.

%% (13) set initial values for searching posterior mode
% para_ini_s(:,y,z) indicates vector of the initial values for finding posterior mode
% for sample y and model z.

% for US
if US==1
para_ini_s(:,:,1)=[  0.5,   0.5,  0.5, 0.5;
                     0.5,   0.5,  0.5, 0.5;
                  2/4, 2/4, 2/4,2/4;
                    9.32,   9.32,   9.32, 9.32;
                     2/3,      2/3,    2/3,    2/3;
                    0.99,   0.99,  0.99,  0.99]; 


%for Japan
elseif US==0
para_ini_s(:,:,1)=[   0.5,    0.5,   0.5;
                     0.5,    0.5, 0.5;
                   2/4, 2/4, 2/4;
                    7.67,   7.67,   7.67;
                     2/3,    2/3,    2/3;
                    0.99,   0.99,   0.99]; 
end

para_ini_s(:,:,2)=para_ini_s(:,:,1);
para_ini_s(:,:,3)=para_ini_s(:,:,1);
para_ini_s(:,:,4)=para_ini_s(:,:,1);
para_ini_s(:,:,5)=para_ini_s(:,:,1);
para_ini_s(:,:,6)=para_ini_s(:,:,1);
para_ini_s(:,:,7)=para_ini_s(:,:,1);
para_ini_s(:,:,8)=para_ini_s(:,:,1);

%% (14) set Instrumental variables



% Z1 is set of IVs for equation1(phillips curve), Z2 is set of IVs for equation2(price distortion law of motion)  
% if it is not needed to include equation2, write 'Z2=[]' (i.e. set Z2 empty).

% *** only ulc-specification GNKPC (model 1) and NKPC (model 2) can include equation 2 ***

% pi_iv(:,x:y) includes from x to y lags of inflation (pi). 

% In the current version, available IVs are:

% pi_iv (inflation, quarterly, %), ulc_iv(100*ln(unit labor cost)),
% gyhat_iv(growth rate of real GDP per capita, quarterly,%)),
% diff_lhat_iv (differenced demeaned 100*ln(hours worked)),
% diff_rhat_iv (differenced demeaned nominal interest rate, quarterly, %),
% ulchat_iv (demeaned 100*log(ulc))
% lhat_iv (demeaned 100*log(hours worked)),
% rhat_iv (demeaned interest rate, quarterly, %),
% diff_pi_iv (changes in inflation),
% yhat_iv (linear detrended 100*log(real GDP per capita) )
% (US only) LSS_iv (long-short spread, quarterly, %),
% (US only) dw_iv (wage growth, quarterly, %),
% (US only) dc_iv (commodity price inflation, quarterly, %).
% If using constant, " ones(length(Y),1) " should be included.

if US==1
    
IV={'Z1=[pi_iv(:,1:4),ulchat_iv(:,1:4),dw_iv(:,1:4),yhat_iv(:,1:4),LSS_iv(:,1:4),dc_iv(:,1:4),ones(length(Y),1)]; Z2=ones(length(Y),1);'};

%IV={'Z1=[pi_iv(:,1:4),ulchat_iv(:,1:2),dw_iv(:,1:2),yhat_iv(:,1:2),ones(length(Y),1)]; Z2=ones(length(Y),1);'};


 
elseif US==0

 IV={'Z1=[pi_iv(:,1:4),ulchat_iv(:,1:2),dw_iv(:,1:2),gyhat_iv(:,1:2),rhat_iv(:,1:2), ones(length(Y),1)]; Z2=ones(length(Y),1);'};

end

%% (15) set AR(1) coef. of transitry TFP innovation
%% 
rho = 0; % dummy;

%% === setting done =====%

%% check setting availability

NumberOfModels=end_estim_model-start_estim_model+1;
NumberOfSample=end_estim_period-start_estim_period+1;

if (size(adjust_c,1)<NumberOfModels)||(size(adjust_c,2)<NumberOfSample)
    disp('Matrix of adjust parameter for MCMC acceptence ratio does not match. Please check again')
    return
end

if (size(para_ini_s,3)<NumberOfModels)||(size(para_ini_s,2)<NumberOfSample)||(size(para_ini_s,1)<6)
    disp('Matrix of initial parameters does not match. Please check again')
    return
end


%% savefile name
%{
if US==1&&cpi==1
    cpi_name='PCE';
elseif US==0&&cpi==1
    cpi_name='CPI';
else
    cpi_name='def';
end
%}

if TSGMM==1
    gmm_name='TSgmm';
elseif TSGMM==0
    gmm_name='CUgmm';
end

if iniWM==1;
    iniWM_name='EYE';
elseif iniWM==0;
    iniWM_name='ZZ';
elseif iniWM==-1
    iniWM_name='dZZ';
end

if fullsamplemean==1
    fsm='fullsamplemean';
else
    fsm='';
end


today=[today1,'K',mat2str(K),gmm_name,'NW',mat2str(NWlags),iniWM_name,'lead',mat2str(lead.f),'lag',mat2str(lead.b),fsm];

%% prior setting
% set parameter matrix and distribution
para_estim0=cell2mat(prior_set(:,1));
%para_ini=cell2mat(prior_set(:,3));
para_pri=cell2mat(prior_set(:,3:4));
dist_name=prior_set(:,2);

% set fixed parameter values
para_fix  =para_pri(:,1); 


%% do MCMC 
tic

num=1;
savefile_root=[pwd(),'\',today1,'\',today,'_BGMMoutput'];
excelfile=[pwd(),'\',today1,'\',today,'mcmc_results_',num2str(num),'.xlsx'];
output_txt=[pwd(),'\',today1,'\(setting)',today,'_',num2str(num),'.m'];

while exist(output_txt)==2
    num=num+1;
    %excelfile=[pwd(),'\',today1,'\',today,'mcmc_results_',num2str(num),'.xlsx'];
    output_txt=[pwd(),'\',today1,'\(setting)',today,'_',num2str(num),'.m'];
end

for model=start_estim_model:end_estim_model
    para_estim=para_estim0;
    if model==2||model==6
        para_estim(4)=0;
    end
    
% creating prior.m file
making_prior        
    
    for sample=start_estim_period:end_estim_period

%% (C) sample period definition

% for US data
if US ==1
%1. Great Inflation: 1966Q1-1982Q3 [77,143]
%2. Post-Great Inflation: 1982Q4-2016Q4 [144, 280]
%3. 1+2: 1966Q1-2016Q4 [77, 280]

if sample==1
    sample_period=[77,143];  
elseif sample==2
    sample_period=[144, 280];
elseif sample==3
    sample_period=[77, 280];

end

%% for comparing Gali&Gertler
% if sample==1
%     sample_period=[53,132];  
% elseif sample==2
%     sample_period=[133, 204];
% elseif sample==3
%     sample_period=[133, 275];
% elseif sample==4
%     sample_period=[133, 242];
% elseif sample==5
%     sample_period= [93, 172];
% elseif sample==6
%     sample_period=[57, 275];
% end


% for Japan
elseif US==0
% 1. moderate: 1981Q4-1997Q4 [47, 111]
% 2. deflation: 1998Q1-2016Q4 [112, 172]
% 3. 1 + 2 : 1981Q4-2016Q4[47, 172]

if sample==1
    sample_period=[47, 111]; % [47, 111]
elseif sample==2
    sample_period=[112, 187]; % [112, 187]
elseif sample==3
    sample_period=[47, 187]; % [47, 187]
end

end
        
        savefile=[savefile_root,'_',num2str(num),'_model',num2str(model),'_sample',num2str(sample)];
        adjust_param=adjust_c(model,sample);
        para_ini=para_ini_s(:,sample,model);
        BGMM_GNKPC
              
    end
end

disp('MCMC for all models and all sample periods, done.')
disp('Now drawing graphs and calculating model selection criterion...')

%% output reduced form coefficients
reducedformcoef

%% Model selection
model_selection

disp('all done!')
toc

%% copy setting file
copyfile([mfilename('fullpath'),'.m'] ,[pwd(),'\',today1,'\(setting)',today,'_',num2str(num),'.m']);

%% clear unnecessary data
clear prior_mat for_graph reduced_savefile posterior_mean post_means HMbox 
clear LAP_s LAPp_s HM5_s HM9_s SWZ5_s SWZ9 CJalt_s MLAP_s MLAPp_s MHM5_s MHM9_s MSWZ5_s MSWZ9_s   MCJalt_s 
clear count_m count_s names disp_post_mean output_post_mean
clear ss mmm sss



%% make sounds
load levelup
sound(y,Fs)