function [W, LAMBDA]=weighting_mat(Zu,NWlags)

%% Quadratic Spectral Weighting Matrix
if NWlags==-2
    [nt,d]=size(Zu);
    %jhat=zeros(d,d);

    % calling the automatic bandwidth selection
    st=bandw2(Zu);

    % lag 0 covariance
    LAMBDA_0T=Zu'*Zu;
    LAMBDA_vT=zeros(d,d);

    for j=1:nt-1
        LAMBDA=Zu(j+1:nt,:)'*Zu(1:nt-j,:);
        LAMBDA_vT=LAMBDA_vT+kernQS(j/st)*(LAMBDA+LAMBDA');
    end
% end
% 
% % backward sum
% for j=1:nt-1
%     jhat=jhat+kern(j/st)*Zu(1:nt-j,:)'*Zu(j+1:nt,:);
% end

    % small sample correction
    LAMBDA=(LAMBDA_vT+LAMBDA_0T)/(nt-d);

%% Bartlett Kernel Weighting Matrix
elseif NWlags==-1
    [nt,d]=size(Zu);
    %jhat=zeros(d,d);

    % calling the automatic bandwidth selection
    st=bandw1(Zu);

    % lag 0 covariance
    LAMBDA_0T=Zu'*Zu;
    LAMBDA_vT=zeros(d,d);
    for j=1:nt-1
        LAMBDA=Zu(j+1:nt,:)'*Zu(1:nt-j,:);
        LAMBDA_vT=LAMBDA_vT+kernBT(j/st)*(LAMBDA+LAMBDA');
    end

    % small sample correction
    LAMBDA=(LAMBDA_vT+LAMBDA_0T)/(nt-d);

%% for Newy-West optimal weighting matrix with a fixed lags
else
    [T,d]=size(Zu);   

    LAMBDA=Zu'*Zu;
    LAMBDA_0T = LAMBDA/T;

    if NWlags==0
        LAMBDA_vT=zeros(d,d);
    else
        LAMBDA_vT=0;

        for n=1:NWlags

            %Zu=Z(n+1:T,:).*repmat(ut(n+1:T),1,n_inst);
            %Zu2=Z(1:T-n,:).*repmat(ut(1:T-n),1,n_inst);

            Zu1=Zu(n+1:end,:);
            Zu2=Zu(1:end-n,:);
            LAMBDA=Zu1'*Zu2;
            LAMBDA=LAMBDA/T;
            LAMBDA=LAMBDA+LAMBDA';
            LAMBDA=(1-n/(NWlags+1))*LAMBDA;
            LAMBDA_vT=LAMBDA_vT+LAMBDA;

        end

    end

    LAMBDA=LAMBDA_0T+LAMBDA_vT;
end

% Weighting matrics
W=eye(d)/LAMBDA;

end

function st=bandw2(vhat) % Autoselect w/ Quadratic spectral kernel 
[nt,d]=size(vhat);
a2n=0;
a2d=0;
for i=1:d
    rho_a=([ones(nt-1,1), vhat(1:nt-1,i)]'*[ones(nt-1,1), vhat(1:nt-1,i)])\([ones(nt-1,1), vhat(1:nt-1,i)]'*vhat(2:nt,i));
    b=rho_a(2);
    sig=(vhat(2:nt,i)-b*vhat(1:nt-1,i))'*(vhat(2:nt,i)-b*vhat(1:nt-1,i));
    sig=sig/(nt-1);
    a2n=a2n+4*b*b*sig*sig/(1-b)^8;
    a2d=a2d+sig*sig/(1-b)^4;
end
a2=a2n/a2d;
st=1.3221*(a2*nt)^.2;
end

function st=bandw1(vhat) % Autoselect w/ Bertlett kernel
[nt,d]=size(vhat);
a1n=0;
a1d=0;
for i=1:d
    rho_a=([ones(nt-1,1), vhat(1:nt-1,i)]'*[ones(nt-1,1), vhat(1:nt-1,i)])\([ones(nt-1,1), vhat(1:nt-1,i)]'*vhat(2:nt,i));
    b=rho_a(2);
    sig=(vhat(2:nt,i)-b*vhat(1:nt-1,i))'*(vhat(2:nt,i)-b*vhat(1:nt-1,i));
    sig=sig/(nt-1);
    a1n=a1n+4*b*b*sig*sig/((1-b)^6*(1+b)^2);
    a1d=a1d+sig*sig/(1-b)^4;
end
a1=a1n/a1d;
st=1.1447*(a1*nt)^.2;
end

function ker=kernQS(x) % QS kernel
del=6*pi*x/5;
ker=3*(sin(del)/del-cos(del))/(del*del);
end

function ker=kernBT(x) % Bertlett kernel
%ker=1-abs(x);
ker=max(1-abs(x),0);
end